// files used: m135_9x.cpp, m135_9x.h
// Lawson Labs, Inc.
// 3239 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800
// April 23, 1998
// Modified from 16-bit EasyWin application with 16-bit DLL.
// This file polled-mode only
// Tim Van Dusen - programmer
// 32-bit Windows-9x console application

#include <condefs.h>
#include <process.h>
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <stdio.h>
#include <alloc.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include <dos.h>
#include "m135_9x.h"
#pragma argsused

UINT      conversion;
T_M135    M135;

int main(int argc, char **argv)
{
  double  stDv=0, rmsBits=0, sdTot = 0, runningAvg=0, sd[41];;
  double  voltage, max=-6, min=6;
  int		  pRow, ch;
  char 	  tmpbuf[80];
  int    	sdCount=1;

	SetWindowToDesktop();

	if (!ReadConfig())
	{
	  gotoxy(24,10);
	  printf("unable to read 135EWIN.CFG file");
	  gotoxy(26,11);
	  printf("press any key to continue");
	  getch();
	}

TOP_MENU:
  ch = 0;

  while(ch != 27)
  {
	  clrscr();
	  gotoxy(27,4);
	  printf ("MODEL 135");
	  gotoxy(27,6);
	  printf ("Main Menu");
	  gotoxy(20,8);
	  printf ("1) ADC              %u",M135.adc135);
	  gotoxy(20,10);
	  printf ("2) Polled Mode Operation");
	  gotoxy(18,14);
	  printf ("Esc) Exit program");

	  ch = getch();

	  if (ch == 27)
	  {
		  goto EXIT_MAIN;
	  }

	  if (ch == '1')
	  {
GET_ADC:
		  clrscr();
		  printf("Enter ADC (check your manual) ");
      getch();
		  fgets(tmpbuf,80,stdin);
		  sscanf(tmpbuf,"%u",&M135.adc135);

		  if (!CheckADC(M135.adc135))
		  {
		    printf ("\n%u is not a valid ADC address \n",M135.adc135);
		    printf("press ENTER to try again");

		    if (getch() == 27)
		    {
			    goto TOP_MENU;
		    }
		    goto GET_ADC;
		  }

		  if (ADInit(M135.adc135, M135.controlCode)==0)
		  {
		    printf ("\nModel 135 not found at ADC %u\n",M135.adc135);
		    printf("press ENTER to try again, ESC to quit");

		    if (getch() == 27)
		    {
			    goto TOP_MENU;
		    }

		    goto GET_ADC;
		  }
	  }

    if (ch == '2')
    {
      if (ADInit(M135.adc135, M135.controlCode)==0)
		  {
		    clrscr();
		    printf ("\nModel 135 not found at ADC %u\n",M135.adc135);
		    printf("press any key to continue");
		    getch();
		    goto TOP_MENU;
		  }

		  goto POLLED;
    }
  } //while Ch != 27

  goto EXIT_MAIN;

POLLED:

  ch = 0;
  pRow = 13;              //starting row to display voltages

  clrscr();
  PrintCommandMenu();
  while (ch != 27)		    //repeat until esc key pressed or an error
  {
    while (!kbhit())      //loop until keyboard hit
	  {
      SetWin32CursorOff();
		  gotoxy(2,pRow++);
		  voltage = GetConv();

		  printf ("  %9.4lf",voltage);

		  if (max < voltage)
		  {
		    max = voltage;
		  }

		  if (min > voltage)
		  {
		    min = voltage;
		  }

		  StandardDev(&stDv, &rmsBits, &sdTot, &runningAvg, sd, &voltage, &sdCount);

		  gotoxy (57,4);
		  if (M135.digInp == 1)
		  {
		    printf("on ");
		  }
		  else
		  {
		    printf("off");
		  }

		  if (pRow == 23)
		  {
		    pRow = 13;

		    gotoxy(40,14);
		    printf ("Maximum     %8.4lf v       ", max);
		    gotoxy (40,15);
		    printf ("Minimum     %8.4lf v       ", min);
		    gotoxy (40,16);
		    printf ("Delta       %8.4lf v       ", fabs(max - min));
		    gotoxy (40,17);
		    printf ("Average     %8.4lf v       ", runningAvg);
		    gotoxy (40,19);
		    printf ("Standard deviation  %8.4lf v    ", stDv);
		    gotoxy (40,20);
		    printf ("Effective Resolution  %5.1lf   bits", rmsBits);
		  }
    }   //while (!kbhit())

	  ch = getch();

	  if (ch != 27)
	  {
		  if (islower(ch))
		  {
		    ch = toupper(ch);
		  }

		  DoCommand (ch, &max, &min);
	  }
    SetWin32CursorOn();
  }	//while (Ch != 27)

  goto TOP_MENU;


EXIT_MAIN:

SAVE_CONFIG:
	clrscr();
	printf("\nDo you wish to save configuration (Y/N)? ");
	ch = getch();

	if (ch == 'Y' || ch == 'y')
	{
	  WriteConfig();
	}
	else if(ch != 'N' && ch != 'n')
	{
	  goto SAVE_CONFIG;
	}

	DestroyWindow(GetActiveWindow());   //close window
	exit(0);
	return 1;
}

void SetWindowToDesktop(void)         //make window the same size as the desktop
{
RECT rect;

  clrscr();
  GetWindowRect(GetDesktopWindow(), &rect);
  SetWindowPos(GetActiveWindow(), (HWND) NULL, 0, 0,
					rect.right, rect.bottom,
					SWP_NOZORDER | SWP_NOACTIVATE);
}

void PrintCommandMenu()
{
  gotoxy (15,1);
  printf ("POLLED  MENU   Lawson Labs Model 135");
  gotoxy (5,3);
  printf ("C) A/D input Channel    %u", (M135.controlCode>>4));
  gotoxy (5,4);
  printf ("E) External sub-channel %u",M135.controlCode&0xF);

  if ((M135.controlCode&0xF) < 10)
  {
	  printf (" ");
  }

  gotoxy (43,3);
  printf ("A) Averaging            %-5u", M135.average);

  gotoxy (43,4); printf ("Digital input ");
  if (M135.digInp == 1)
  {
	  printf("on ");
  }
  else
  {
	  printf("off");
  }

  gotoxy (7,11); printf ("Volts\n");
  gotoxy (1,24);
  printf (" Press ESC for menu                                              ");
}

void DoCommand(int ch, double *max, double *min)
{
  int	tempCh, tempExt;
  char 	tmpbuf[80];

  if (ch == 'A')
  {
	  M135.average = GetAvg(M135.average);
  }

  if (ch == 'C')
  {
    tempCh = -1;
    gotoxy(1,24);
    printf("                                              ");

	  while ((tempCh < '0') || (tempCh > '3'))
	  {
      gotoxy(1,24);
      printf("Enter channel ");
      tempCh =(short)getche();
	  }

	  M135.controlCode = ((short)(tempCh - '0')<<4) + (M135.controlCode&0xF);

	  gotoxy(1,24);
	  printf("                                              ");
  }

  if (ch == 'E')
  {
    tempExt = -1;
    gotoxy(1,24);
    printf("                                              ");

	  while ((tempExt < 0) || (tempExt > 15))
	  {
      gotoxy(1,24);
      printf("Enter sub channel                           ");
      gotoxy(19,24);
      getch();
      fgets(tmpbuf,80,stdin);
      sscanf(tmpbuf,"%d",&tempExt);
	  }

	  M135.controlCode = ((M135.controlCode>>4)<<4) + tempExt;
	  printf("                                              ");
  }

  if (ch == ' ')
  {
    (*max) = -6.0;
    (*min) = 6.0;             //reset max-min
  }

  PrintCommandMenu();
}

UINT GetAvg(UINT average)
{
  int tempCh = 0;

  while((tempCh != 27) && (tempCh != 13))
  {
	  while (!kbhit())
	  {
      gotoxy (1,24);
      printf ( "Press + increase, - decrease averaging, Enter to continue ");
      printf ( "%-6u", average);
	  }

	  tempCh =(short) getch();
	  if (tempCh == '+')
	  {
		  average *= 2;
		  if (average == 0)
		  {
		    average = 1;
		  }
	  }

	  if (tempCh == '-')
	  {
		  average /= 2;
	  }

	  if (average  < 1)
	  {
		  average = 256;
	  }
	  else if (average > 256)
	  {
		  average = 1;
	  }
  }
  return average;
}

short ReadConfig()
{
  int n;
  char tmpbuf[80];
  FILE *rCfg;

	if ((rCfg = fopen("135ewin.cfg", "rt")) != NULL)
	{
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.adc135);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lf",&M135.hertzRate);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.average);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.ichDelay);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.numData);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lu",&M135.totScans);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.controlCode);
	  for (n = 0;n<= 65;n++)
	  {
		  fgets(tmpbuf,80,rCfg);
		  sscanf(tmpbuf,"%u",&M135.cCode[n]);
	  }
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M135.timeStamp);

	  fclose(rCfg);
	  return 1;
	}
	else
	{
	  M135.adc135 = 600;
	  M135.hertzRate = 10000;
	  M135.ichDelay = 0;
	  M135.average = 1;
	  M135.numData = 1;
	  M135.totScans = 100;
	  M135.controlCode = 0;
	  for (n=0;n<=65;n++)
	  {
		  M135.cCode[n] = 0;
	  }
	  M135.timeStamp = 0;

	  return 0;
	}
}


short WriteConfig()
{
  int n;
  FILE *wCfg;

  if ((wCfg = fopen("135ewin.cfg", "wt")) != NULL)
  {
    fprintf(wCfg,"%u\n",M135.adc135);
    fprintf(wCfg,"%lf\n",M135.hertzRate);
    fprintf(wCfg,"%u\n",M135.average);
    fprintf(wCfg,"%u\n",M135.ichDelay);
    fprintf(wCfg,"%u\n",M135.numData);
    fprintf(wCfg,"%lu\n",M135.totScans);
    fprintf(wCfg,"%u\n",M135.controlCode);
    for (n = 0;n<= 65;n++)
	  {
		  fprintf(wCfg,"%u\n",M135.cCode[n]);
	  }
	  fprintf(wCfg,"%u\n",M135.timeStamp);
	  fclose(wCfg);
	  return 1;
  }

  return 0;
}


short CheckADC(UINT adc135)		//check for valid ADC
{
  if (adc135 == 544 ||
		  adc135 == 552 ||
		  adc135 == 560 ||
		  adc135 == 568 ||
		  adc135 == 576 ||
		  adc135 == 584 ||
		  adc135 == 592 ||
		  adc135 == 600 ||
		  adc135 == 608 ||
		  adc135 == 616 ||
		  adc135 == 624)
  {
	  return 1;
  }
  return 0;
}

//standard deviation and RMS resolution calculation
void    StandardDev(double *stDv, double *rmsBits, double *sdTot,
                    double *runningAvg, double *sd, double *voltage,
						        int *sdCount)
{
  int i = 0, nnn, sdGrp=40;;
  double tempStDv, StDvRms;

  sd[*sdCount] = (*voltage);
  (*sdTot) += (*voltage);

  if ((*sdCount) == sdGrp)
  {
    (*stDv) = 0; (*sdTot) = (*sdTot) / sdGrp;

    for (nnn = 1; nnn != sdGrp; nnn++)
    {
    (*stDv) = (*stDv) + pow(((*sdTot) - sd[nnn]), 2);
    }

    (*stDv) = sqrt((*stDv) / sdGrp);
    StDvRms = (*stDv) / 1.52588E-04;  //.001192093;
    tempStDv = StDvRms;

CALCNEXT:

    if (tempStDv >= 1)
    {
      i = i + 1;
      tempStDv = tempStDv / 2;
      goto CALCNEXT;
    }

    (*rmsBits) = 16 - i;

    if ((tempStDv < .75) && (i != 0))
    {
      (*rmsBits) = (*rmsBits) + .5;
    }

    (*runningAvg) = (*sdTot);
    (*sdTot) = 0; (*sdCount) = 1;
  }
  else
  {
	 (*sdCount)++;
  }
}

short ADInit(int adc, BYTE cCode)
{
  outp(adc, cCode);	                    //current channel
  inp(adc + 2);	                        //start conv
  return ReadStatus(adc);
}

double GetConv()
{
  static  UINT	adc135, tempavg, start;
  static  ULONG avgConv;
  BYTE    nextChan;
  double  v=0;

  tempavg = 1;
  adc135 = M135.adc135;
  start = adc135 + 1;
  nextChan = M135.controlCode;
  avgConv = 0;

Get_Conv_Again:
  if (ReadStatus(adc135)==0)
  {
	 	goto Get_Conv_Error;
  }
  _asm mov edx, start;
  __emit__(0xed);       // in
  asm{
      and eax, 0ffffh
      xor ah, 80h
      mov conversion, eax
      mov al, nextChan
      dec dx
  }
  __emit__(0xee);       // out

  avgConv = avgConv + conversion;

  if (tempavg++ != M135.average)
  {
		goto Get_Conv_Again;
  }

  if (inp(adc135)&128)
  {
	 	M135.digInp = 0;
  }
  else
  {
	 	M135.digInp = 1;
  }
  v = ((avgConv/M135.average)-32768.0) * 1.52588E-04;

  return v;

Get_Conv_Error:
  return -10;
}

short ReadStatus(UINT status)
{
  unsigned short readStatus;

  _asm  xor ax, ax;
  _asm  mov readStatus, 1;
  _asm  mov edx, status;
  _asm  mov cx, 512;
READ_STATUS:
  __emit__(0xec);
  _asm  and al, 02h;
  _asm  jnz STATUS_END;
  _asm  loop READ_STATUS;
  _asm  mov readStatus, 0;
STATUS_END:

  return readStatus;
}

void outp(UINT outPort, int outVal)
{
  _asm mov eax, outVal;
  _asm mov edx, outPort;
  __emit__(0xef);
}

short inp(UINT iVal)
{
  _asm mov edx, iVal;
  __emit__(0xec);
  return _AX;
}

void SetWin32CursorOn(void)
{
	ShowCursor(TRUE);
	_setcursortype(_NORMALCURSOR);
}

void SetWin32CursorOff(void)
{
	ShowCursor(FALSE);
	_setcursortype(_NOCURSOR);
}


